/**********************************************************************
ID: decompose.c

Copyright 2001, Bulent Yilmaz

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

#include "decompose.h"

char *getpart(char *part) {

    char *partstart = 0;
    char *retstr = 0;

#if TESTOUTPUT   
    cflog("Inside getpart.");
#endif
    
    partstart=strchr(part, ':');
    // Get past the spaces.
    while(*(++partstart)==' ');
    
    retstr=strdup(partstart);

#if TESTOUTPUT       
    fprintf(getloghandle(), "Leaving getpart:\n(%d)---->%s<----\n",retstr,retstr);
#endif

    return retstr;
}

int cleanmsgparts(MSGPARTS *inparts) {

    inparts->approved=NULL;
    inparts->content_base=NULL;
    inparts->content_disposition=NULL;
    inparts->content_type=NULL;
    inparts->control=NULL;
    inparts->date=NULL;
    inparts->distribution=NULL;
    inparts->followup_to=NULL;
    inparts->from=NULL;
    inparts->lines=0;
    inparts->message_id=NULL;
    inparts->newsgroups=NULL;
    inparts->nntp_posting_host=NULL;
    inparts->organization=NULL;
    inparts->path=NULL;
    inparts->references=NULL;
    inparts->reply_to=NULL;
    inparts->sender=NULL;
    inparts->subject=NULL;
    inparts->x_trace=NULL;
    inparts->x_newsreader=NULL;
    inparts->x_newsposter=NULL;
    inparts->x_mailer=NULL;
    inparts->x_cancelled_by=NULL;
    inparts->body=NULL;
    
    return 0;
}

int freemsgparts(MSGPARTS *inparts) {

#if TESTOUTPUT
    cflog("freemsgparts: About to free all the strings");
#endif

    if(inparts->approved!=NULL) free(inparts->approved);
    if(inparts->content_base!=NULL) free(inparts->content_base);
    if(inparts->content_disposition!=NULL) free(inparts->content_disposition);
    if(inparts->content_type!=NULL) free(inparts->content_type);
    if(inparts->control!=NULL) free(inparts->control);
    if(inparts->date!=NULL) free(inparts->date);
    if(inparts->distribution!=NULL) free(inparts->distribution);
    if(inparts->followup_to!=NULL) free(inparts->followup_to);
    if(inparts->from!=NULL) free(inparts->from);
    if(inparts->message_id!=NULL) free(inparts->message_id);
    if(inparts->newsgroups!=NULL) free(inparts->newsgroups);
    if(inparts->nntp_posting_host!=NULL) free(inparts->nntp_posting_host);
    if(inparts->organization!=NULL) free(inparts->organization);
    if(inparts->path!=NULL) free(inparts->path);
    if(inparts->references!=NULL) free(inparts->references);
    if(inparts->reply_to!=NULL) free(inparts->reply_to);
    if(inparts->sender!=NULL) free(inparts->sender);
    if(inparts->subject!=NULL) free(inparts->subject);
    if(inparts->x_trace!=NULL) free(inparts->x_trace);
    if(inparts->x_newsreader!=NULL) free(inparts->x_newsreader);
    if(inparts->x_newsposter!=NULL) free(inparts->x_newsposter);
    if(inparts->x_mailer!=NULL) free(inparts->x_mailer);
    if(inparts->x_cancelled_by!=NULL) free(inparts->x_cancelled_by);
    inparts->body=NULL;

#if TESTOUTPUT
    cflog("freemsgparts: Just freed all the strings");
#endif
    
    return 0;
}

int decompose(char *inmsg, MSGPARTS *mymsg, long int msglen) {
	
    char partbuf[2048];
    long int offset = 0;
    long int partoffset = 0;
    char *tmp;
    
    cleanmsgparts(mymsg);
    
#if TESTOUTPUT
    cflog("In decompose: Entering Loop");
#endif
    
    while((*(inmsg+offset)!='\n')&&(*(inmsg+offset+1)!='\n')&&(offset<msglen)) {
        // Buffer overflow just waiting to happen in next line.
        // Will fix later.

#if TESTOUTPUT
        cflog("In decompose: Top Loop");
#endif
        
        partbuf[0]=0;
        while((*(inmsg+offset)!='\n')&&(offset<msglen)&&((offset-partoffset)<2047)) {
        	partbuf[(offset-partoffset)]=*(inmsg+offset);	
        	offset++;
        }
        partbuf[(offset-partoffset)]=0;

#if TESTOUTPUT
        cflog("In decompose: After Little Loop");
#endif
        
        if((mymsg->approved==NULL)&&strstr(&partbuf[0],"Approved:")) mymsg->approved = getpart(partbuf);
        else if((mymsg->content_base==NULL)&&strstr(&partbuf[0],"Content-Base:")) mymsg->content_base = getpart(partbuf);
        else if((mymsg->content_disposition==NULL)&&strstr(&partbuf[0],"Content-Disposition:")) mymsg->content_disposition = getpart(partbuf);    
        else if((mymsg->content_type==NULL)&&strstr(&partbuf[0],"Content-Type:")) mymsg->content_type = getpart(partbuf);    
        else if((mymsg->control==NULL)&&strstr(&partbuf[0],"Control:")) mymsg->control = getpart(partbuf);    
        else if((mymsg->date==NULL)&&strstr(&partbuf[0],"Date:")) mymsg->date = getpart(partbuf);
        else if((mymsg->distribution==NULL)&&strstr(&partbuf[0],"Distribution:")) mymsg->distribution = getpart(partbuf);    
        else if((mymsg->followup_to==NULL)&&strstr(&partbuf[0],"Followup-To:")) mymsg->followup_to = getpart(partbuf);
        else if((mymsg->from==NULL)&&strstr(&partbuf[0],"From:")) mymsg->from = getpart(partbuf);    
        else if((mymsg->message_id==NULL)&&strstr(&partbuf[0],"Message-ID:")) mymsg->message_id = getpart(partbuf);
        else if((mymsg->newsgroups==NULL)&&strstr(&partbuf[0],"Newsgroups:")) mymsg->newsgroups = getpart(partbuf);    
        else if((mymsg->nntp_posting_host==NULL)&&strstr(&partbuf[0],"NNTP-Posting-Host:")) mymsg->nntp_posting_host = getpart(partbuf);
        else if((mymsg->organization==NULL)&&strstr(&partbuf[0],"Organization:")) mymsg->organization = getpart(partbuf);    
        else if((mymsg->path==NULL)&&strstr(&partbuf[0],"Path:")) mymsg->path = getpart(partbuf);
        else if((mymsg->references==NULL)&&strstr(&partbuf[0],"References:")) mymsg->references = getpart(partbuf);    
        else if((mymsg->reply_to==NULL)&&strstr(&partbuf[0],"Reply-To:")) mymsg->reply_to = getpart(partbuf);
        else if((mymsg->sender==NULL)&&strstr(&partbuf[0],"Sender:")) mymsg->sender = getpart(partbuf);    
        else if((mymsg->subject==NULL)&&strstr(&partbuf[0],"Subject:")) mymsg->subject = getpart(partbuf);
        else if((mymsg->x_trace==NULL)&&strstr(&partbuf[0],"X-Trace:")) mymsg->x_trace = getpart(partbuf);    
        else if((mymsg->x_newsreader==NULL)&&strstr(&partbuf[0],"X-Newsreader:")) mymsg->x_newsreader = getpart(partbuf);
        else if((mymsg->x_newsposter==NULL)&&strstr(&partbuf[0],"X-Newsposter:")) mymsg->x_newsposter = getpart(partbuf);    
        else if((mymsg->x_mailer==NULL)&&strstr(&partbuf[0],"X-Mailer:")) mymsg->x_mailer = getpart(partbuf);
        else if((mymsg->x_cancelled_by==NULL)&&strstr(&partbuf[0],"X-Cancelled-By:")) mymsg->x_cancelled_by = getpart(partbuf);    
        else if((mymsg->x_cancelled_by==NULL)&&strstr(&partbuf[0],"X-Canceled-By:")) mymsg->x_cancelled_by = getpart(partbuf);
        else if((mymsg->lines==0)&&strstr(&partbuf[0],"Lines:")) {
            tmp=getpart(partbuf);
            mymsg->lines=atoi(tmp);
            free(tmp);
        }
#if TESTOUTPUT
        else {
        	fprintf(getloghandle(), "No matching headers:\n***->%s<-***\n",partbuf);
        }
#endif
        
        offset++;
        partoffset=offset;
    }
    
#if TESTOUTPUT
    if(offset>=msglen) cflog("Decompose came out at end of message.");
#endif    
    
    mymsg->body=inmsg+offset;

#if TESTOUTPUT
    cflog("Leaving decompose");
#endif
      
    return 0;
} 


