/**********************************************************************
ID: filters.c

Copyright 2001, Bulent Yilmaz

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

#include "filters.h"

#if BLOCK_BINARIES

int ismessagebinary(MSGPARTS *mymsg) {

    static regex_t ismessagebinary_base64,ismessagebinary_uuencode;
    static int ismessagebinary_compiled=0;
    int err;
    char errbuf[255];
    
#if TESTOUTPUT
    cflog("ismessagebinary: About to see if regex_t is compiled");
#endif

    if(!ismessagebinary_compiled) {
#if TESTOUTPUT
        cflog("ismessagebinary: About to compile regex_t");
#endif
        if((err=regcomp(&ismessagebinary_base64,
                        "(^[\t>]*[A-Za-z0-9\+\/]{59,76}[\t ]*\r?\n){"
                        MAX_ENCODED_LINES "}",
                    REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   ))==0) {
#if TESTOUTPUT
	    cflog("ismessagebinary: Compiled ismessagebinary_base64");
#endif	    
        } else {
#if TESTOUTPUT
            regerror(err, &ismessagebinary_base64, &errbuf, 255);
            cflog("ismessagebinary: Error in base64");
#endif
            return 0;
        }

        if((err=regcomp(&ismessagebinary_uuencode,
                        "(^[\t>]*M[\x20-\x60]{60,61}[\t ]*\r?\n){"
                        MAX_ENCODED_LINES "}",
                    REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   ))==0) {
#if TESTOUTPUT
	    cflog("ismessagebinary: Compiled ismessagebinary_uuencode");
#endif	    
        } else {
#if TESTOUTPUT
            regerror(err, &ismessagebinary_uuencode, &errbuf, 255);
            cflog("ismessagebinary: Error in uuencode");
#endif
            return 0;
        }
        ismessagebinary_compiled=1;
    }

    if( mymsg->lines > MAX_ENCODED_LINES_I ) {
#if TESTOUTPUT
        cflog("ismessagebinary: More than MAX_ENCODED_LINES, testing regexps");
#endif
        if(!regexec(&ismessagebinary_base64, mymsg->body, 0, NULL, REG_NEWLINE)) {
#if TESTOUTPUT
            cflog("ismessagebinary: Message matches base64 binary");
#endif
            return 1;
        } else if(!regexec(&ismessagebinary_uuencode, mymsg->body, 0, NULL, REG_NEWLINE)) {
            cflog("ismessagebinary: Message matches uuencoded binary");
            return 1;
        }
    }    
    return 0;
}

int groupsbinarymatch(char *filtertext) {

    static regex_t groupsbinarymatch_allowed_groups;
    static int groupsbinarymatch_compiled=0;
    char *start,*end;
    char group[255];

#if TESTOUTPUT
    cflog("groupsbinarymatch: About to see if regex_t is compiled");
#endif
    
    if(!groupsbinarymatch_compiled) {
#if TESTOUTPUT
        cflog("groupsbinarymatch: About to compile regex_t");
#endif
        if(!regcomp(&groupsbinarymatch_allowed_groups,
                     "\\.binae?r|^alt\\.sex\\.pictures|^fur\\.artwork"
                     "|multimedia|alt\\.anonymous\\.messages|^de\\.alt\\.dateien"
                     "|^rec\\.games\\.bolo$|^comp\\.security\\.pgp\\.test$"
                     "|sfnet\\.tiedostot",
                    REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {
#if TESTOUTPUT
            cflog("groupsbinarymatch: Regexp loaded");
#endif
            groupsbinarymatch_compiled=1;
        } else return -1;    
    }

#if TESTOUTPUT
    cflog("groupsbinarymatch: Regexp is loaded");
#endif
    
    start=filtertext;
    while((end=strchr(start,','))) {
        strncpy(group,start,(end-start));
        group[(end-start-1)]=0;
#if TESTOUTPUT
        cflog("groupsbinarymatch: About to test (inside loop)");
#endif
        if(regexec(&groupsbinarymatch_allowed_groups,group,0,NULL,REG_NEWLINE)) return 1;
	start=end+1;
    }
    
#if TESTOUTPUT
        cflog("groupsbinarymatch: About to test (outside loop)");
#endif

    if(regexec(&groupsbinarymatch_allowed_groups,start,0,NULL,REG_NEWLINE)) return 1;

    return 0;
}
#endif

#if DO_BOT_CHECKS
int botsfilter(MSGPARTS *mymsg) {

    static regex_t emailplatinum,twopointoxbot,newsgroupbulkmailer,calvacade98;
    static regex_t atomicpost,botmessageidpattern,orgmidbotpattern,msgidabcxyzbot;
    static regex_t nonepoundyetbot,strippathbot,adultsightsbot_org;
    static regex_t adultsightsbot_msgid,repostbot;
    static int botsfilter_compiled=0;
    
#if TESTOUTPUT
    cflog("botsfilter: About to see if regex_t is compiled");
#endif

    if(!botsfilter_compiled) {
        if(!regcomp(&emailplatinum, "email\s+platinum",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {
#if TESTOUTPUT
            cflog("botsfilter: emailplatinum Regexp loaded");
            
#endif
        } else return 0;

#if TESTOUTPUT
        cflog("botsfilter: about to load twopointoxbot");
        
#endif
                        
        if(!regcomp(&twopointoxbot, "^2\.\d\.(\d\d? [A-Z]|\d\d?)$",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: twopointobot Regexp loaded");
            
#endif
        } else return 0;

#if TESTOUTPUT
        cflog("botsfilter: about to load newsgroupbulkmailer");
        
#endif        

        if(!regcomp(&newsgroupbulkmailer, "newsgroup\s+bulk\s+mailer",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: newsgroupbulkmailer Regexp loaded");
            
#endif
        } else return 0;
        
        if(!regcomp(&calvacade98, "calvacade 98",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: calvacade98 Regexp loaded");
            
#endif
        } else return 0;        

        if(!regcomp(&atomicpost, "atomicpost",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: atomicpost Regexp loaded");
            
#endif
        } else return 0;        

        if(!regcomp(&botmessageidpattern, "<\d[12]\@[A-Z]{10}>|\@\d+>",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: botmessageidpattern Regexp loaded");
            
#endif
        } else return 0;        

        if(!regcomp(&orgmidbotpattern, "<no\s+organization>",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: orgmidbotpattern Regexp loaded");
            
#endif
        } else return 0;

        if(!regcomp(&msgidabcxyzbot, "msgidabcxyz\.com",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: msgidabcxyzbot Regexp loaded");
            
#endif
        } else return 0;        

        if(!regcomp(&nonepoundyetbot, "none\d+\.yet>",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: nonepoundyetbot Regexp loaded");
            
#endif
        } else return 0;        
        
        if(!regcomp(&strippathbot, "strip_path>",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: strippathbot Regexp loaded");
            
#endif
        } else return 0;

        if(!regcomp(&adultsightsbot_org, "^[A-Z]{2,3}\sInc\.\s*$",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: adultsightsbot_org Regexp loaded");
            
#endif
        } else return 0;

        if(!regcomp(&adultsightsbot_msgid, "<\d{12}\.\d{10}\@",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: adultsightsbot_msgid Regexp loaded");
            
#endif
        } else return 0;

        if(!regcomp(&repostbot, "repost.*unauthorized.*cancel",
                        REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {                                                                                                                
#if TESTOUTPUT
            cflog("botsfilter: repostbot Regexp loaded");
            
#endif
        } else return 0;
        
#if TESTOUTPUT
        cflog("botsfilter: All patterns loaded");
                
#endif
        botsfilter_compiled=1;        
    }

#if TESTOUTPUT
    cflog("botsfilter: About to test patterns");
    
#endif

    if(mymsg->organization) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for organization");
        
#endif
        
        if(!regexec(&emailplatinum,mymsg->organization,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&repostbot,mymsg->organization,0,NULL,REG_NEWLINE)) return 1;
    }
    if(mymsg->x_newsreader) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for x_newsreader");
        
#endif

        if(!regexec(&twopointoxbot,mymsg->x_newsreader,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&newsgroupbulkmailer,mymsg->x_newsreader,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&calvacade98,mymsg->x_newsreader,0,NULL,REG_NEWLINE)) return 1;
    }
    if(mymsg->x_newsposter) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for x_newsposter");
        
#endif
        if(!regexec(&atomicpost,mymsg->x_newsposter,0,NULL,REG_NEWLINE)) return 1;
    }
    if(mymsg->message_id) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for message_id");
        
#endif

        if(!regexec(&botmessageidpattern,mymsg->message_id,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&nonepoundyetbot,mymsg->message_id,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&strippathbot,mymsg->message_id,0,NULL,REG_NEWLINE)) return 1;
        if(!regexec(&nonepoundyetbot,mymsg->message_id,0,NULL,REG_NEWLINE)) return 1;
    }
    if((mymsg->organization)&&(mymsg->message_id)) {

#if TESTOUTPUT
        cflog("botsfilter: About to test patterns for AdultSights");
        
#endif

        if((!regexec(&adultsightsbot_org,mymsg->organization,0,NULL,REG_NEWLINE))&&
           (!regexec(&adultsightsbot_msgid,mymsg->message_id,0,NULL,REG_NEWLINE))) return 1;
    }
    return 0;
}
#endif

#if PATH_MATCH
int pathmatch(char *msgpath) {

    static regex_t matching_path;
    static int matching_path_compiled=0;
    FILE *temphandle;
    char *pathregex;

#if TESTOUTPUT
    cflog("pathmatch: About to see if regex_t is compiled");
#endif
    
    if(!matching_path_compiled) {
#if TESTOUTPUT
      cflog("pathmatch: About to compile regex_t");
#endif
      if((pathregex=getautoacceptregex()) != 0) {
        if(!regcomp(&matching_path,
                    pathregex,
                    REG_EXTENDED|REG_NOSUB|REG_NEWLINE
                   )) {
#if TESTOUTPUT
            cflog("pathmatch: Regexp loaded");
#endif
	    free(pathregex);
            matching_path_compiled=1;
        } else {
            free(pathregex);
            return -1;    
        }
      }
    }

#if TESTOUTPUT
    temphandle=getloghandle();
    fprintf(temphandle, "About to compare %s to REGEXP %s\n", msgpath, PATH_REGEX);
    fflush(temphandle);
#endif

    if(!regexec(&matching_path,msgpath,0,NULL,REG_NEWLINE)) {
#if TESTOUTPUT
	cflog("pathmatch: Path matched PATH_REGEX");
#endif
	return 1;
    }
    
    return 0;
}
#endif