/**********************************************************************
ID: usieve.c

Copyright 2001, Bulent Yilmaz

Part of Microbrew MicroSieve
USENET SPAM filter

http://www.microbrew.org/products/usieve.html


MicroSieve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**********************************************************************/

#include <errno.h>
#include "usieve.h"

int main() {

    char *inbuf=0;
    long int read_index=0, newread=0, max_art_size=0;
    MSGPARTS currmsg;
    int reject=0,eom=0;

#if STATLOGGING
    long int articles=0, begintime=0, nowtime=0;
    unsigned long int bytes=0;
#endif

#if TESTOUTPUT
    FILE *temphandle;
#endif

#if TESTOUTPUT
    cflog("Starting program.");
#endif

    writepid();
    
    max_art_size=getmaxartsize();
    
    inbuf=malloc(max_art_size);
    if(!inbuf) {
       cflog("Could not allocate buffer for articles.  Ending program.");
       return -1;
    }
    
//  Magic:

    setvbuf(stdin, NULL, _IONBF, 0);

    while(1) {
      reject=0;
      read_index=0;
      eom=0;

      *inbuf=0;
      while(!eom) {
#if TESTOUTPUT
          cflog("Trying to do a read");
#endif
#if TESTOUTPUT
          temphandle=getloghandle();
          fprintf(temphandle, "Already %ld bytes in buffer.\n",read_index);
          fflush(temphandle);
#endif
	  if(feof(stdin)) {
	      clearerr(stdin);
#if TESTOUTPUT
              cflog("Cleared error on stdin");
#endif          	      
	  }
#if TESTOUTPUT
          temphandle=getloghandle();
          fprintf(temphandle, "Trying to read %ld bytes.\n",(max_art_size-read_index));
          fflush(temphandle);
#endif                              
	  newread=read(STDIN_FILENO, (inbuf + read_index), (max_art_size-read_index));
#if TESTOUTPUT
          temphandle=getloghandle();
          fprintf(temphandle, "Just read %ld bytes.\n",newread);
          if (newread < 0) {
              fprintf(temphandle, "read returned errorno %d\n", errno);
          }
          fflush(temphandle);
#endif
	  read_index+=newread;
          *(inbuf+read_index)=0;

          if((*(inbuf+read_index-1)=='\n')&&(*(inbuf+read_index-2)=='\r')&&(*(inbuf+read_index-3)=='.')&&(*(inbuf+read_index-4)=='\n')) {
	      eom=1;
#if TESTOUTPUT
	      temphandle=getloghandle();
	      fprintf(temphandle, "Article: %ld bytes.\n",read_index);
	      fflush(temphandle);
#endif
          } else if( read_index >= max_art_size ) {	
          
              // If the message is greater than MAX_ART_SIZE:
              
              eom=1;
              reject=1;
              
              // Read to end of article and go on.
              
              while( !((*(inbuf+read_index-1)=='\n')&&(*(inbuf+read_index-2)=='\r')&&(*(inbuf+read_index-3)=='.')&&(*(inbuf+read_index-4)=='\n')) ) {
#if TESTOUTPUT
		  cflog("Clearing out end of input buffer");
#endif
                  read_index=read(STDIN_FILENO, inbuf, max_art_size);
              }
#if TESTOUTPUT
	      cflog("Message too big.  Rejecting...");
#endif
          }
         
      }
      
      if((read_index > 5)&&(!reject)) {
        
#if TESTOUTPUT
        cflog("Just read article");
        temphandle=getloghandle();
        fprintf(temphandle, "Article Length: %ld:\n",read_index);
	fflush(temphandle);
#endif
#if LOGLASTARTICLE
	logarticle(inbuf,read_index);
#endif		        
        decompose(inbuf,&currmsg,read_index);
#if TESTOUTPUT
        fprintf(temphandle, "Out of decompose.\n");
        fflush(temphandle);
        if(currmsg.approved) fprintf(temphandle, "1: %s\n",currmsg.approved);
        if(currmsg.content_base) fprintf(temphandle, "2: %s\n",currmsg.content_base);
        if(currmsg.content_disposition) fprintf(temphandle, "3: %s\n",currmsg.content_disposition);
        if(currmsg.content_type) fprintf(temphandle, "4: %s\n",currmsg.content_type);
        if(currmsg.control) fprintf(temphandle, "5: %s\n",currmsg.control);
        if(currmsg.date) fprintf(temphandle, "6: %s\n",currmsg.date);
        if(currmsg.distribution) fprintf(temphandle, "7: %s\n",currmsg.distribution);
        if(currmsg.followup_to) fprintf(temphandle, "8: %s\n",currmsg.followup_to);
        if(currmsg.from) fprintf(temphandle, "9: %s\n",currmsg.from);
        if(currmsg.lines) fprintf(temphandle, "10: %d\n",currmsg.lines);
        if(currmsg.message_id) fprintf(temphandle, "11: %s\n",currmsg.message_id);
        if(currmsg.newsgroups) fprintf(temphandle, "12: %s\n",currmsg.newsgroups);
        if(currmsg.nntp_posting_host) fprintf(temphandle, "13: %s\n",currmsg.nntp_posting_host);
        if(currmsg.organization) fprintf(temphandle, "14: %s\n",currmsg.organization);
        if(currmsg.path) fprintf(temphandle, "15: %s\n",currmsg.path);
        if(currmsg.references) fprintf(temphandle, "16: %s\n",currmsg.references);
        if(currmsg.reply_to) fprintf(temphandle, "17: %s\n",currmsg.reply_to);
        if(currmsg.sender) fprintf(temphandle, "18: %s\n",currmsg.sender);
        if(currmsg.subject) fprintf(temphandle, "19: %s\n",currmsg.subject);
        if(currmsg.x_trace) fprintf(temphandle, "20: %s\n",currmsg.x_trace);
        if(currmsg.x_newsreader) fprintf(temphandle, "21: %s\n",currmsg.x_newsreader);
        if(currmsg.x_newsposter) fprintf(temphandle, "22: %s\n",currmsg.x_newsposter);
        if(currmsg.x_mailer) fprintf(temphandle, "23: %s\n",currmsg.x_mailer);
        if(currmsg.x_cancelled_by) fprintf(temphandle, "24: %s\n",currmsg.x_cancelled_by);
        fflush(temphandle);
#endif

#if PATH_MATCH
	if(currmsg.path) {
	 if(!reject) {
#if TESTOUTPUT
	 temphandle=getloghandle();
	 fprintf(temphandle, "About to check path: %s", currmsg.path);
	 fflush(temphandle);
#endif	 
	  if(!pathmatch(currmsg.path)) {
#if TESTOUTPUT
	   cflog("Path did not match.  Performing other checks");
#endif
#endif

#if BLOCK_BINARIES
	   if(!reject) {
    	    if(currmsg.newsgroups) {
                if(groupsbinarymatch(currmsg.newsgroups)) {
#if TESTOUTPUT
        	    cflog("Message is not in binary group... checking for binary in message");
#endif        	    
                    if(ismessagebinary(&currmsg)) {
                        reject=1;
#if TESTOUTPUT                        
                        cflog("Message is binary in non-binary group. Rejecting...");
#endif                        
		    }
                }
            }
           }
#endif

#if DO_BOT_CHECKS
           if(!reject) {
            if(botsfilter(&currmsg)) {
#if TESTOUTPUT            
                cflog("Bot detected.  Rejecting...");
#endif                
                reject=1;        	
    	    }
           }
#endif

#if PATH_MATCH
	  }
#if TESTOUTPUT
	  else cflog("Path Matched.  Auto-accepting Message.");
#endif
	 }
	}
#endif
      }
      if(reject) {
          fputs(REJECTCODE, stdout);
          cflog("Article Rejected");
      }
      else {
          fputs(ACCEPTCODE, stdout);
          cflog("Article Passed.");
      }
      fflush(stdout);
        
      freemsgparts(&currmsg);
      
      
#if STATLOGGING
      articles++;
      bytes+=read_index;
      if (articles==1) begintime=(time(0)-1);
      temphandle=getloghandle();
      nowtime=time(0);
      fprintf(temphandle, "Art: %ld \tKBytes: %lu \tSeconds: %ld \tArt/Min %d\n",articles,(bytes/1024),(nowtime-begintime),((articles*60)/(nowtime-begintime)));
      fflush(temphandle);
#endif

#if (STATLOGGING && TESTOUTPUT)
      if((articles%25000)==24999) cflogreopen();
#endif
    } 

    puts("Exited loop.  This shouldn'ta happened!");
    return 0;

}
